/* $NetBSD$ */

/*-
 * Copyright (c)2008, 2011 Citrus Project,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/cdefs.h>
#if defined(LIBC_SCCS) && !defined(lint)
__RCSID("$NetBSD$");
#endif /* LIBC_SCCS and not lint */

#include <assert.h>
#include <errno.h>
#include <stddef.h>
#include <stdint.h>
#include <wchar.h>

#include "xlocale.h"
#include "setlocale_local.h"

#include "citrus_module.h"
#include "citrus_ctype.h"
#include "runetype_local.h"
#include "multibyte.h"

#define _RUNE_LOCALE(locale) \
    ((_RuneLocale *)(_locale_to_impl(locale))->part_impl[(size_t)LC_CTYPE])

#define _CITRUS_CTYPE(locale) \
    (_RUNE_LOCALE(locale)->rl_citrus_ctype)

size_t
_mbrlen_l(const char *s, size_t n, mbstate_t *ps, _locale_t locale)
{
	size_t ret;
	int err0;

	_fixup_ps(_RUNE_LOCALE(locale), ps, s == NULL);

	err0 = _citrus_ctype_mbrlen(_ps_to_ctype(ps), s, n,
				     _ps_to_private(ps), &ret);
	if (err0)
		errno = err0;

	return ret;
}

int
_mbsinit_l(const mbstate_t *ps, _locale_t locale)
{
	int ret;
	int err0;
	_RuneLocale *rl;

	if (ps == NULL)
		return 1;

	if (_ps_to_runelocale(ps) == NULL)
		rl = _RUNE_LOCALE(locale);
	else
		rl = _ps_to_runelocale(ps);

	/* mbsinit should cause no error... */
	err0 = _citrus_ctype_mbsinit(rl->rl_citrus_ctype,
				      _ps_to_private_const(ps), &ret);
	if (err0)
		errno = err0;

	return ret;
}

size_t
_mbrtowc_l(wchar_t *pwc, const char *s, size_t n, mbstate_t *ps, _locale_t locale)
{
	size_t ret;
	int err0;

	_fixup_ps(_RUNE_LOCALE(locale), ps, s == NULL);

	err0 = _citrus_ctype_mbrtowc(_ps_to_ctype(ps), pwc, s, n,
				       _ps_to_private(ps), &ret);
	if (err0)
		errno = err0;

	return ret;
}

size_t
_mbsrtowcs_l(wchar_t *pwcs, const char **s, size_t n, mbstate_t *ps,
    _locale_t locale)
{
	size_t ret;
	int err0;

	_fixup_ps(_RUNE_LOCALE(locale), ps, s == NULL);

	err0 = _citrus_ctype_mbsrtowcs(_ps_to_ctype(ps), pwcs, s, n,
					_ps_to_private(ps), &ret);
	if (err0)
		errno = err0;

	return ret;
}

size_t
_wcrtomb_l(char *s, wchar_t wc, mbstate_t *ps, _locale_t locale)
{
	size_t ret;
	int err0;

	_fixup_ps(_RUNE_LOCALE(locale), ps, s == NULL);

	err0 = _citrus_ctype_wcrtomb(_ps_to_ctype(ps), s, wc,
				       _ps_to_private(ps), &ret);
	if (err0)
		errno = err0;

	return ret;
}

size_t
_wcsrtombs_l(char *s, const wchar_t **ppwcs, size_t n, mbstate_t *ps,
    _locale_t locale)
{
	size_t ret;
	int err0;

	_fixup_ps(_RUNE_LOCALE(locale), ps, s == NULL);

	err0 = _citrus_ctype_wcsrtombs(_ps_to_ctype(ps), s, ppwcs, n,
					_ps_to_private(ps), &ret);
	if (err0)
		errno = err0;

	return ret;
}

wint_t
_btowc_l(int c, _locale_t locale)
{
	wint_t ret;
	int err0;

	err0 = _citrus_ctype_btowc(_CITRUS_CTYPE(locale), c, &ret);
	if (err0)
		errno = err0;

	return ret;
}

int
_wctob_l(wint_t wc, _locale_t locale)
{
	int ret;
	int err0;

	err0 = _citrus_ctype_wctob(_CITRUS_CTYPE(locale), wc, &ret);
	if (err0)
		errno = err0;

	return ret;
}
