/* $NetBSD$ */

/*-
 * Copyright (c)2008, 2011 Citrus Project,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/cdefs.h>
#if defined(LIBC_SCCS) && !defined(lint)
__RCSID("$NetBSD$");
#endif /* LIBC_SCCS and not lint */

#include <assert.h>
#include <string.h>

#include "xlocale.h"
#include "setlocale_local.h"

const char *
_querylocale(int mask, _locale_t locale)
{
	struct _locale_impl_t *impl;
	int howmany, categories[_LC_LAST - 1], i, j;
	_locale_category_t *l;
	const char *s, *t;

	impl = _locale_to_impl(locale);
	howmany = 0;
	for (i = 1; i < _LC_LAST; ++i) {
		if (mask & (1 << i))
			categories[howmany++] = i;
	}
	impl->query[0] = '\0';
	if (howmany > 0) {
		l = _find_category(categories[0]);
		_DIAGASSERT(l != NULL);
		s = (*l->setlocale)(NULL, impl);
		_DIAGASSERT(s != NULL);
		strlcat(&impl->query[0], s, sizeof(impl->query));
		for (i = 1, j = 0; i < howmany; ++i) {
			l = _find_category(categories[i]);
			_DIAGASSERT(l != NULL);
			t = (*l->setlocale)(NULL, impl);
			_DIAGASSERT(t != NULL);
			if (j == 0) {
				if (!strcmp(s, t))
					continue;
				for (j = 1; j < i; ++j) {
					strlcat(&impl->query[0], "/",
					    sizeof(impl->query));
					strlcat(&impl->query[0], s,
					    sizeof(impl->query));
				}
			}
			strlcat(&impl->query[0], "/",
			    sizeof(impl->query));
			strlcat(&impl->query[0], t,
			    sizeof(impl->query));
		}
	}
	return (const char *)&impl->query[0];
}
