#include <ctype.h>
#include <errno.h>
#include <langinfo.h>
#include <limits.h>
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "xlocale.h"

#define NL_LANGINFO_L(item, locale) nl_langinfo(item)
#include "t_nl_langinfo.h"
void
test_nl_langinfo()
{
	TEST_NL_LANGINFO_L;
}
#undef NL_LANGINFO_L
#undef TEST_NL_LANGINFO_L

#define NL_LANGINFO_L(item, locale) _nl_langinfo_l(item, locale)
#include "t_nl_langinfo.h"
void
test_nl_langinfo_l(_locale_t locale)
{
	TEST_NL_LANGINFO_L;
}

#define LOCALECONV_L(locale)	localeconv()
#include "t_localeconv.h"
void
test_localeconv()
{
	TEST_LOCALECONV_L;
}
#undef LOCALECONV_L
#undef TEST_LOCALECONV_L

#define LOCALECONV_L(locale)	_localeconv_l(locale)
#include "t_localeconv.h"
void
test_localeconv_l(_locale_t locale)
{
	TEST_LOCALECONV_L;
}

void
test_multibyte_l(_locale_t locale, const char *s, size_t n)
{
        mbstate_t st;
        wchar_t wc;

	fprintf(stderr, "---- %s ----\n", __func__);
	fprintf(stderr, "encoding: %s\n", _nl_langinfo_l(CODESET, locale));
	fprintf(stderr, "mb_cur_max:%d\n", _MB_CUR_MAX_L(locale));
	fprintf(stderr, "---- c90 ----\n");
	fprintf(stderr, "stateful:%zd\n", _mbtowc_l(NULL, NULL, 0, locale));
	fprintf(stderr, "converted:%zd\n", _mbtowc_l(&wc, s, n, locale));
	fprintf(stderr, "wc:%#x\n", wc);

	fprintf(stderr, "---- amd1 ----\n");
	_mbrtowc_l(NULL, NULL, 0, &st, locale);
	fprintf(stderr, "converted:%zd\n", _mbrtowc_l(&wc, s, n, &st, locale));
	fprintf(stderr, "wc:%#x\n", wc);

	fprintf(stderr, "---- wctype ----\n");
	fprintf(stderr, "width:%d\n", _wcwidth_l(wc, locale));
	fprintf(stderr, "graph:%d\n", _iswgraph_l(wc, locale));
}

int
main(void)
{
	_locale_t locale0, locale1;
	char s0[] = "\xa4\xa2";		/* eucJP */
	char s1[] = "\xe3\x81\x81";	/* UTF-8 */

	fprintf(stderr, " %s\n", setlocale(LC_ALL, ""));
	test_localeconv();
	test_nl_langinfo();

	setlocale(LC_ALL, "");
	fprintf(stderr, "***** %s\n", setlocale(LC_ALL, NULL));

	locale0 = _newlocale(_LC_ALL_MASK, "ja_JP.eucJP", NULL);
	fprintf(stderr, "***** %s\n", _querylocale(_LC_ALL_MASK, locale0));

	locale1 = _newlocale(_LC_ALL_MASK, "ja_JP.UTF-8", NULL);
	fprintf(stderr, "***** %s\n", _querylocale(_LC_ALL_MASK, locale1));

	test_localeconv();
	test_localeconv_l(locale0);
	test_localeconv_l(locale1);

	test_nl_langinfo();
	test_nl_langinfo_l(locale0);
	test_nl_langinfo_l(locale1);

	test_multibyte_l(locale0, s0, sizeof(s0));
	test_multibyte_l(locale1, s1, sizeof(s1));

	_uselocale(locale0);
	fprintf(stderr, "%d\n", _freelocale(NULL));
	fprintf(stderr, "%d\n", _freelocale(_LC_GLOBAL_LOCALE));
	fprintf(stderr, "%d\n", _freelocale(locale0));
	fprintf(stderr, "%d\n", _freelocale(locale1));
	return 0;
}
