/* $NetBSD$ */

/*-
 * Copyright (c)2008, 2011 Citrus Project,
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef _XLOCALE_H_
#define _XLOCALE_H_

#include <ctype.h>
#include <langinfo.h>
#include <locale.h>
#include <wctype.h>
#include <wchar.h>

typedef struct _locale_impl_t	*_locale_t;

/* locale.h */
#define _LC_COLLATE_MASK	((int)(1 << LC_COLLATE))
#define _LC_CTYPE_MASK		((int)(1 << LC_CTYPE))
#define _LC_MONETARY_MASK	((int)(1 << LC_MONETARY))
#define _LC_NUMERIC_MASK	((int)(1 << LC_NUMERIC))
#define _LC_TIME_MASK		((int)(1 << LC_TIME))
#define _LC_MESSAGES_MASK	((int)(1 << LC_MESSAGES))
#define _LC_ALL_MASK		((int)~0)
#define _LC_GLOBAL_LOCALE	((_locale_t)-1)

__BEGIN_DECLS

/* ctype.h */
int		_isalnum_l(int, _locale_t);
int		_isalpha_l(int, _locale_t);
int		_isblank_l(int, _locale_t);
int		_iscntrl_l(int, _locale_t);
int		_isdigit_l(int, _locale_t);
int		_isgraph_l(int, _locale_t);
int		_islower_l(int, _locale_t);
int		_isprint_l(int, _locale_t);
int		_ispunct_l(int, _locale_t);
int		_isspace_l(int, _locale_t);
int		_isupper_l(int, _locale_t);
int		_isxdigit_l(int, _locale_t);
int		_toupper_l(int, _locale_t);
int		_tolower_l(int, _locale_t);

/* langinfo.h */
char		*_nl_langinfo_l(nl_item, _locale_t);

/* locale.h */
_locale_t	_newlocale(int, const char *, _locale_t);
int		_freelocale(_locale_t);
_locale_t	_duplocale(_locale_t);
const char	*_querylocale(int, _locale_t);
_locale_t	_uselocale(_locale_t);
struct lconv	*_localeconv_l(_locale_t);

/* stdlib.h */
int		_mblen_l(const char *, size_t, _locale_t);
int		_mbtowc_l(wchar_t * __restrict, const char * __restrict,
		    size_t, _locale_t);
size_t		_mbstowcs_l(wchar_t * __restrict, const char * __restrict,
		    size_t, _locale_t);
int		_wctomb_l(char *, wchar_t, _locale_t);
size_t		_wcstombs_l(char * __restrict, const wchar_t * __restrict,
		    size_t, _locale_t);
size_t		__get_mb_cur_max_l(_locale_t);

/* wchar.h */
wint_t		_btowc_l(int, _locale_t);
int		_wctob_l(wint_t, _locale_t);
int		_mbsinit_l(const mbstate_t *, _locale_t);
size_t		_mbrlen_l(const char * __restrict, size_t,
		    mbstate_t * __restrict, _locale_t);
size_t		_mbrtowc_l(wchar_t * __restrict, const char * __restrict,
		    size_t, mbstate_t *, _locale_t);
size_t		_mbsrtowcs_l(wchar_t * __restrict, const char ** __restrict,
		    size_t, mbstate_t * __restrict, _locale_t);
size_t		_wcrtomb_l(char * __restrict, wchar_t wc,
		    mbstate_t * __restrict, _locale_t);
size_t		_wcsrtombs_l(char * __restrict, const wchar_t ** __restrict,
		    size_t, mbstate_t * __restrict, _locale_t);
int		_wcwidth_l(wchar_t, _locale_t);
int		_wcswidth_l(const wchar_t *, size_t, _locale_t);


/* wctype.h */
int		_iswalnum_l(wint_t, _locale_t);
int		_iswalpha_l(wint_t, _locale_t);
int		_iswblank_l(wint_t, _locale_t);
int		_iswcntrl_l(wint_t, _locale_t);
int		_iswdigit_l(wint_t, _locale_t);
int		_iswgraph_l(wint_t, _locale_t);
int		_iswlower_l(wint_t, _locale_t);
int		_iswprint_l(wint_t, _locale_t);
int		_iswpunct_l(wint_t, _locale_t);
int		_iswspace_l(wint_t, _locale_t);
int		_iswupper_l(wint_t, _locale_t);
int		_iswxdigit_l(wint_t, _locale_t);
wint_t		_towupper_l(wint_t, _locale_t);
wint_t		_towlower_l(wint_t, _locale_t);
wctype_t	_wctype_l(const char *, _locale_t);
wctrans_t	_wctrans_l(const char *, _locale_t);
int		_iswctype_l(wint_t, wctype_t, _locale_t);
wint_t		_towctrans_l(wint_t, wctrans_t, _locale_t);

__END_DECLS

#define _MB_CUR_MAX_L(locale) __get_mb_cur_max_l(locale)

#endif /*_XLOCALE_H_*/
